/*------------------------------------------------------------------------
 |  FILE            ImageCanvas.java
 |  PART OF         SHH SpaceGame - a simple, networked multiplayer game.
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

package util;

import java.net.*;
import java.awt.*;

/**
 *
 *
 * @author    Sverre H. Huseby
 *            &lt;<A HREF="mailto:shh@thathost.com">shh@thathost.com</A>&gt;
 * @version   $Id: ImageCanvas.java,v 1.3 2002/01/27 14:47:33 sverrehu Exp $
 */
public class ImageCanvas
extends Canvas {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private int   width;
    private int   height;
    private Image img;

    private void waitForImage(Image img) {
        MediaTracker mt;

        mt = new MediaTracker(this);
        mt.addImage(img, 0);
        try {
            mt.waitForID(0);
        } catch (InterruptedException e) {
        }
    }

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    public ImageCanvas(Image img, int width, int height) {
        this.width = width;
        this.height = height;
        this.img = img;
        waitForImage(img);
    }

    public ImageCanvas(String resource, int width, int height) {
        URL url;

        if ((url = getClass().getResource(resource)) == null) {
            System.err.println("unable to get resource `" + resource + "'");
            return;
        }
        this.width = width;
        this.height = height;
        img = Toolkit.getDefaultToolkit().getImage(url);
        waitForImage(img);
    }

    public ImageCanvas(URL url, int width, int height) {
        this.width = width;
        this.height = height;
        img = Toolkit.getDefaultToolkit().getImage(url);
        waitForImage(img);
    }

    /* Canvas ***********************************************************/
    public void paint(Graphics g) {
        if (img != null)
            g.drawImage(img, 0, 0, this);
    }

    public Dimension getMinimumSize() {
        return new Dimension(width, height);
    }

    public Dimension getPreferredSize() {
        return getMinimumSize();
    }
}
